package jp.co.sds_corp.fxchart.graph;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.util.Iterator;
import java.util.Map;

import javax.imageio.ImageIO;

import jp.co.sds_corp.nanchart.graph.DefaultStorage;
import jp.co.sds_corp.nanchart.graph.Graph2D;
import jp.co.sds_corp.nanchart.graph.StairGraph;
import jp.co.sds_corp.nanchart.graph.XScale;
import jp.co.sds_corp.nanchart.graph.YScale;
import jp.co.sds_corp.util.common.CmnDate;

/**
* Tickグラフを生成します。
*/
public class RateGraphTick extends RateGraph
{
	protected void makeGraph( OutputStream out )
		throws Exception
	{
		CmnDate to = getTo();
		if ( to == null ) {
			to = new CmnDate();
			setTo( to );
		}
		CmnDate fr = getFrom();
		if ( fr == null ) {
			fr = (CmnDate )to.clone();
			fr.add( CmnDate.MINUTE, -30 );
			setFrom( fr );
		}

		Map<String,RateHistRosoku> mapHist = getSource().getData( fr, to, span );

		double	dMax = getSource().getMax();
		double	dMin = getSource().getMin();

/*
		if ( mapHist.size() == 0 ) {
			RateExchange re = new RateExchange( pair );
			re.loadCurrentRate();
			dMin = dMax = re.getFwdRate( "SP" ).mid.doubleValue();
		}
*/
		double  dDiv = Math.pow( 10, pair.getPoint() - 1 );
		dMin = ( Math.floor( dMin*dDiv )) / dDiv ;
		dMax = ( Math.floor( dMax*dDiv ) + 1.0 ) / dDiv ;

		Graph2D grf = new Graph2D( fr, to, new Double( dMin ), new Double( dMax ));
		drawGraph( grf, fr, to, dMin, dMax, mapHist );
		XScale  xs = (XScale )grf.getXScale();
		setXScale( xs, span, fr, to );
		YScale  ys = (YScale )grf.getYScale();
		setYScale( ys, dMin, dMax, dDiv, pair );

		Rectangle rect = new Rectangle( 35, 5, getWidth()-getMarginWidth()-10,
										getHeight()-getMarginHeight() );
		grf.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
								RenderingHints.VALUE_ANTIALIAS_ON);
		if ( getBgImage() != null ) {
			grf.setBgImage( getBgImage() );
		}
		BufferedImage img;
		img = grf.getImage( getWidth(), getHeight(), rect, BufferedImage.TYPE_INT_RGB );
		copyright( img, 0, getHeight() - 10 );

		ImageIO.write( img, "jpeg", out );
	}

	void drawGraph( Graph2D grf, CmnDate fr, CmnDate to,
			double dMin, double dMax, Map<String,RateHistRosoku> mapHist )
	{
		StairGraph dat = new StairGraph( fr, to,
						new BigDecimal( dMin ), new BigDecimal( dMax ));
		dat.setStroke(
			new BasicStroke( 1, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL ));
		DefaultStorage dt = (DefaultStorage)dat.getStorage();
		grf.addSeries( dat );
		dat.setLineColor( Color.black );

		RateHistRosoku cur = null;
		Iterator<String> itr = mapHist.keySet().iterator();
		while ( itr.hasNext()) {
			cur = (RateHistRosoku )mapHist.get( itr.next() );
			dt.add( cur.getStartDate(), cur.getStart() );
		}
		if ( cur != null ) {
			dt.add( new CmnDate(), cur.getStart() );
		}

	}

}
